#!/usr/bin/perl
#
# $B%i%$%V%i%j$rMQ$$$F(B, $B%P!<%8%g%sHV9f$J$I$r<hF@$9$k%3!<%I(B. 
#
# $Id: fk-grubver.pl,v 1.12 2005/12/13 02:32:00 hi-lo Exp $
use lib '../lib';

use lib '../conf';

use FKSConfig;
use misc;

use strict;

my $usage = qq{Usage: $0 [Options] target
target: IP or Hostname
Options:
  -h        show this message.
  -o        output logfile. (default: STDOUT)
            except grub log of each port. 
  -b        output base file name for each port. (default: STDOUT)
            "-b outfile" makes "outfile.port.log". 
  -t        target TCP/UDP port number(s). separeted with ',(comma)'.
            (ex: -t 25,80,110,443,990-1000 )
  -u        open TCP/UDP port number(s). separeted with ',(comma)'. 
  -c        closed TCP/UDP port number(s). separeted with ',(comma)'.
  -f        filtered TCP/UDP port number(s). separeted with ',(comma)'.
  -s        grub version information with using SSL.
  -T        TCP scan (Default)
  -U        UDP scan

};

# $B%3%^%s%I%i%$%sA4BN$rJ]B8(B. 
my $command = "$0";
for my $i (@ARGV){
  $command .=" $i";
}

# $B%*%W%7%g%s$N2r@O(B
use Getopt::Std;
use vars qw($opt_o $opt_t $opt_b $opt_h $opt_s $opt_u $opt_c $opt_f $opt_T $opt_U);

getopts('o:t:u:f:c:b:hsTU') or die $usage;

die $usage if $opt_h;

# $BBP>]%[%9%HL>(B(IP$B%"%I%l%9(B)
my $target = $ARGV[0];

# $BBP>]%]!<%H$N;XDj$,$J$1$l$P%*%W%7%g%s$rI=<($7$F=*N;(B
#if (! $opt_t){
#  print "no target ports are specified.\n";
#  die $usage;
#}

my $target_ports = join (',', expand_port_range($opt_t));
my $open_ports = $opt_u ? $opt_u : $no_ports;
$open_ports = join (',', expand_port_range($open_ports));
my $closed_ports = $opt_c ? $opt_c : $no_ports;
$closed_ports = join (',', expand_port_range($closed_ports));
my $filtered_ports = $opt_f ? $opt_f : $no_ports;
$filtered_ports = join (',', expand_port_range($filtered_ports));

# $BBP>]%[%9%H$N;XDj$,$J$1$l$P%*%W%7%g%s$rI=<($7$F=*N;(B
if (! $target){
  print "no target is specified.\n";
  die $usage;
}

# o$B%*%W%7%g%s$,$"$l$P%"%&%H%W%C%H@h$r;XDj(B. 
my $OUT = \*STDOUT;

if ($opt_o){
  open(LOG, ">> $opt_o") or die "Can't open $opt_o: $!";
  $OUT = \*LOG;
}

# $BBP>]%]!<%H$N;XDj$r8!:w(B 
my @target_ports = split /,/, $target_ports;
#if (! ($#target_ports >= 0)){
#  print $OUT eval(fk_err) . "no valid target ports are specified.\n";
#  print $usage;
#  close LOG if ($opt_o);
#  exit 1;
#}

# $B%9%-%c%s$O%G%U%)%k%H$G(BTCP
my %port_cmd = %tcp_port_cmd;
my $port_type = "TCP";
# -U$B%*%W%7%g%s$,$"$l$P(BUDP$BMQ$N%9%-%c%s(B
if ($opt_U){
  %port_cmd = %udp_port_cmd;
  $port_type = "UDP";
}

#$B$G$-$l$P(B, $B%3%^%s%I%i%$%sA4BN$rF~$l$?$$(B. 
#$B;CDj(B
print $OUT "$command\n";

print $OUT "=====BEGIN=====Target: $target=====" . `date` ;

my @target_ports= sort {$a <=> $b} @target_ports;

for my $target_port ('first' , @target_ports , 'all_host', 'last'){
  next if ($target_port =~ /^\s*$/ );

  # $B?t;z(B or $BJ8;zNs$N;XDj$,$"$k(B
  my $target_svc = $target_port;
  
  if ($target_port =~ /^\d+$/){
    # $B?t;z$N>l9g$O(B, $B$H$j$"$($:(Bunknown$B$H$$$&%5!<%S%9L>$r$D$1$k(B
    $target_svc = "unknown";
    # $B%]!<%HHV9f$,(B, $B%5!<%S%9L>$K%"%5%$%s$5$l$F$$$k>l9g$O(B
    # $B%5!<%S%9L>$r<hF@(B. 
    $target_svc = $port_name{$target_port} if (exists $port_name{$target_port});
  }

  my $PORT_OUT = $OUT;
  # $B8F$S=P$7$?3F%D!<%k$N%m%0$N=q$-=P$7@h(B
  if ($opt_b){
    open(PORT_LOG, ">> $opt_b.$target_port.log") or die "Can't open $opt_b: $!";
    $PORT_OUT = \*PORT_LOG;
  }
  
  my $date_tmp = `date`;
  chomp $date_tmp;
  my $message_tmp;
  if ($target_port =~ /^\d+$/){
    # $B?t;z$N>l9g$O(B, $B%]!<%HHV9f(B/[TCP or UDP]$B$r$D$1$k(B
    $message_tmp = "##grub information for $target_svc($target_port/$port_type) @ $date_tmp ##"; 
  }else{
    # $B;XDj$5$l$?%]!<%HHV9f$,J8;zNs$N>l9g$O(B, [TCP or UDP]$B$O$D$1$J$$(B
    $message_tmp = "##do with signature for '$target_svc' @ $date_tmp##";
  }
    
  print $OUT "\n" x 5 ;
  print $OUT "#" x length($message_tmp);
  print $OUT "\n";
  print $OUT "$message_tmp\n";
  print $OUT "#" x length($message_tmp);

  for my $grub_tool (@{$port_cmd{$target_svc}} ){
    my $date_str =  `date`;
    chomp $date_str;
    print $PORT_OUT "\n[$date_str] $grub_tool\n";
    
    # _OPEN_PORTS_ _CLOSED_PORTS_ _FILTERED_PORTS_ ... $B$O(B, $B$=$l$>$lCV49$9$k(B. 
    # $BCV49$9$kCM$O(B, $B%3%s%^(B(,)$B$G6h@Z$C$?%]!<%HHV9f$N%j%9%H(B. ($BNc(B: "25,80,443")
    if ($grub_tool =~ /_TARGET_PORTS_/ && $target_ports == undef){
      print $OUT eval(fk_err) . "no valid target ports are specified.\naborted !\n";
      next;
    }
    $grub_tool =~ s/_TARGET_PORTS_/$target_ports/go;
    
    if ($grub_tool =~ /_OPEN_PORTS_/ && $open_ports == undef){
      print $OUT eval(fk_err) . "no valid open ports are specified.\naborted !\n";
      next;
    }
    $grub_tool =~ s/_OPEN_PORTS_/$open_ports/go;

    if ($grub_tool =~ /_CLOSE_PORTS_/ && $closed_ports == undef){
      print $OUT eval(fk_err) . "no valid target ports are specified.\naborted !\n";
      next;
    }
    $grub_tool =~ s/_CLOSE_PORTS_/$closed_ports/go;

    if ($grub_tool =~ /_FILTERED_PORTS_/ && $filtered_ports == undef){
      print $OUT eval(fk_err) . "no valid filtered ports are specified.\naborted !\n";
      next;
    }
    $grub_tool =~ s/_FILTERED_PORTS_/$filtered_ports/go;

    # $B%?!<%2%C%H$H$J$k%[%9%H$K$D$$$F$O(B, $B>e$G4{$K3NG':Q$_(B.
    $grub_tool =~ s/_TARGET_HOST_/$target/go;

    my $grub_cmd = "$libexec/$grub_tool -p $target_port $target";
    $date_str =  `date`;
    chomp $date_str;
    print $PORT_OUT "\n[$date_str] $grub_cmd\n";
    eval {
      my @result = `$grub_cmd`;
      for my $line (@result){
        print $PORT_OUT $line ;
      }
    }
  }

  close (PORT_LOG) if ($opt_b);
}

print $OUT "\n=====END=====Target: $target=====" . `date` ;

close(LOG) if ($opt_o);

exit 0;


